;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: org.langband.vanilla -*-

#|

Based on:

DESC: variants/vanilla/levels.lisp - level customisation for Vanilla
Copyright (c) 2000-2002 - Stig Erik Sand

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

|#

(in-package :org.langband.vanilla)


(defun icfp-create-bare-board-level-obj ()
  "Returns a bare board-level."
  (make-instance 'icfp-board-level :depth 0 :rating 0))


(defmethod level-ready? ((level icfp-board-level))
  (when (level.dungeon level)
    t))


(defvar *first-server-response*)

(defun terminate-with-error (error-text)
  ;; FIXME
  (format *error-output* "~&~%Protocol error in server communication:~&~A~%"
          error-text)
  (ext:quit))

(defmethod generate-level! ((variant icfp-variant) (level icfp-board-level) player)
  "Generates a town and returns it.  If the dungeon
argument is non-NIL it will be re-used and returned as
part of the new level."

  (handler-case (join-game)
    (error (cond)
      (terminate-with-error (format nil "Error joining game: ~A" cond))))

  (let* ((*level* level)
;;	 (var-obj *variant*)
	 (town-width *width*)
	 (town-height *height*)
	 (dungeon (create-dungeon town-width
				  town-height
				  :its-depth 0))
	 (screen-width *screen-width*)
	 (screen-height *screen-height*)
	 (qy 0)
	 (qx 0)
	 )

    (declare (type u-fixnum qy qx))

;;    (warn "Generating town with random state equal to ~s" *random-state*)
    
;;    (setf (player.map player) (make-map dungeon))


    (fill-dungeon-with-floor! dungeon +floor-regular+)
    
    (let ((table (dungeon.table dungeon)))
      (dotimes (y town-height)
        (dotimes (x town-width)
          (let ((coord (aref table x y)))
            (setf (coord-floor coord)
                  (ecase (sq-char (get-sq x y))
                    (#\# +floor-perm-solid+)
                    (#\. +floor-regular+)
                    ;;; FIXME.
                    (#\@ +floor-glyph+)
                    (#\~ +floor-water+)))))))

    
    (let ((dungeon-height town-height)
	  (dungeon-width town-width))
      ;;(warn "H: ~s W: ~s" dungeon-height dungeon-width)
      ;; perm walls on top and bottom
      (loop for y in (list 0 (1- dungeon-height))
	    for x from 0 below dungeon-width
	    do
	    (setf (cave-floor dungeon x y) +floor-perm-solid+))
      
      ;; perm walls on left and right
      (loop for x in (list 0 (1- dungeon-width))
	    for y from 0 below dungeon-height
	    do
	    (setf (cave-floor dungeon x y) +floor-perm-solid+)))
    
    (setf (level.dungeon level) dungeon)

    (handler-case
        (progn
          (setq *first-server-response* (get-line))
          (extract-initial-player-positions dungeon player
                                            *first-server-response*))
      (error (cond)
        (terminate-with-error (format nil "Error joining game: ~A" cond))))
    
    (setq *rounds-completed* 0)
;    (format *debug-io*
;            "~&~@<I am ~A with $~A and a capacity of ~A.~:@>~%"
;            *self* *maxmoney* *maxweight*)
#||    
    ;; testing
    (let* ((mon (get-monster-kind variant "robot"))
	   (amon (produce-active-monster variant mon)))
      (place-single-monster! dungeon player amon 5 5 nil))

    (let* ((mon (get-object-kind variant "package"))
	   (amon (create-aobj-from-kind mon :variant variant :amount 1)))
      (let-floor-carry! dungeon 7 7 amon))
||#

;;    (setf (level.dungeon level) dungeon)


    level))

(defmethod do-stuff-after-game-is-first-displayed ((var-obj icfp-variant)
                                                   dungeon
                                                   player)
  ;; Errors in this first call are internal program errors.
  (process-action-report dungeon player *first-server-response*)
  (handler-case
      (process-pkg-report dungeon (get-line))
    (error (cond)
      (terminate-with-error (format nil "Error joining game: ~A" cond)))))


(defun icfp-make-board-level-obj (variant player)
  "A sucky function which should be simplified greatly."

  (flet ((do-generation (seed)
	   (let* ((builder (get-level-builder "board-level"))
		  (town (funcall builder))
		  (cl:*random-state* (cl:make-random-state seed)))

	     (generate-level! variant town player))))
  
    ;; we already have a saved value
    (cond ((variant.town-seed variant)
	   (do-generation (variant.town-seed variant)))
	 
	  ;; this is the first time
	  (t
	   (let* ((new-state (cl:make-random-state t)))
	     (setf (variant.town-seed variant) new-state)
	     (do-generation new-state)))
	  )))


(defmethod create-appropriate-level ((variant icfp-variant) old-level player depth)

  (declare (ignore old-level))
  (let ((level nil))

    (assert (= depth 0))
    (setf level (icfp-make-board-level-obj variant player))
    
    ;; we set the depth now.
    (setf (level.depth level) depth)

    (unless (level-ready? level)
      ;; (warn "Generating level ~a" level)
      (generate-level! variant level player))


    (assert (level-ready? level))
    
    level))



(defmethod activate-object :after ((level icfp-board-level) &key)
  
  (let* ((dungeon (level.dungeon level))
	 (player *player*)
	 (var-obj *variant*)
	 (turn (variant.turn var-obj))
	 (mod-time (mod turn (variant.day-length var-obj)))
	 ;; quick hack
	 (time-of-day (if (or t (< mod-time (variant.twilight var-obj)))
			  'day
			  'night)))

    ;; hackish
    (when nil
    (let ((resident-num (if (eq time-of-day 'day) 4 8)))
	
      ;; add some inhabitants
      (dotimes (i resident-num)
	(allocate-monster! var-obj dungeon player 3 t))))

    (van-town-illuminate! dungeon player time-of-day)
    ;;(warn "post activate")
    ))


(defun van-town-illuminate! (dungeon player time-of-day)
  "Illuminates the town according to the time-of-day."
  (declare (ignore player))
  
    (with-dungeon (dungeon (coord x y))
      (declare (ignore x y))
      (let ((feat (coord.floor coord)))
	(bit-flag-add! (coord.flags coord) #.(logior +cave-glow+ +cave-mark+
						     +cave-seen+))))

    (bit-flag-add! *update* +pl-upd-forget-view+ +pl-upd-update-view+)
    (bit-flag-add! *redraw* +print-map+)


  )
	 

;; this one does the job for all.. only one table for objects in icfp
(defmethod get-otype-table ((var-obj icfp-variant) level)
  (declare (ignore level))
  (%get-var-table var-obj "level" 'objects-by-level))

(defmethod get-mtype-table ((var-obj icfp-variant) (level random-level))
  (%get-var-table var-obj level 'monsters-by-level))

(defmethod get-mtype-table ((var-obj icfp-variant) (level icfp-board-level))
  (%get-var-table var-obj level 'monsters-by-level))

;; when we pass a string
(defmethod get-mtype-table ((var-obj icfp-variant) (level string))
  (%get-var-table var-obj level 'monsters-by-level))
