;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: org.langband.vanilla -*-

#||

Based on:

DESC: variants/vanilla/print.lisp - printout code for vanilla
Copyright (c) 2002 - Stig Erik Sand

This program is free software  ; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation	 ; either version 2 of the License, or
(at your option) any later version.

||#

(in-package :org.langband.vanilla)

(defmethod print-depth ((level level) (setting bottom-row-locations))
  "prints current depth somewhere"
  (let ((column (slot-value setting 'depth)))
    (with-foreign-str (s)
      (lb-format s "~d ft" (* 50 (level.depth level)))
      (put-coloured-str! +term-l-blue+ s column
			 (get-last-console-line)) 
      )))


(defmethod print-depth ((level icfp-board-level) (setting bottom-row-locations))
  "prints current depth somewhere"
  ;; do nothing.
  (when nil
    (let ((column (slot-value setting 'depth)))
      (put-coloured-str! +term-l-blue+ "Town"
                         column
                         (get-last-console-line)))))

(defmethod print-cut ((variant icfp-variant) (player player) 
                      (setting icfp-basic-frame-locations))

  (let* ((cuts (get-attribute-value '<cut> (player.temp-attrs player)))
         (loc (slot-value setting 'cut))
         (cut-info (%get-cutlvl cuts)))

;;    (warn "print cut ~s ~s" cuts cut-info) 
    
    (put-coloured-str! (second cut-info) (third cut-info)
                       (cdr loc) (car loc))
    ))


(defmethod print-stun ((variant icfp-variant) (player player) 
                      (setting icfp-basic-frame-locations))
  
  (let* ((stun (get-attribute-value '<stun> (player.temp-attrs player)))
         (loc (slot-value setting 'stun))
         (stun-info (%get-stunlvl stun)))
    
    (put-coloured-str! (second stun-info) (third stun-info)
                       (cdr loc) (car loc))
    ))

(defmethod print-poisoned ((variant icfp-variant) (player player) 
                           (setting icfp-bottom-row-locations))

  (let ((column (slot-value setting 'poisoned))
        (row (get-last-console-line)))

    (if (get-attribute-value '<poisoned> (player.temp-attrs player))
        (put-coloured-str! +term-orange+ "Poisoned" column row)
        (put-coloured-str! +term-white+  "        " column row))
    ))

(defmethod print-state ((variant icfp-variant) (player player) 
			(setting icfp-bottom-row-locations))

  (let ((word nil)
	(colour +term-white+)
	(temp-attrs (player.temp-attrs player))
	(column (slot-value setting 'state))
        (row (get-last-console-line)))

    (cond ((get-attribute-value '<paralysed> temp-attrs)
	   (setf word "Paralysed!"
		 colour +term-red+))
	  ;; turn it off
	  (t
	   (setf word "          ")
	   ))

    (put-coloured-str! colour word column row)
    
    t))

(defmethod print-afraid ((variant icfp-variant) (player player) 
			 (setting icfp-bottom-row-locations))

  (let ((column (slot-value setting 'afraid))
        (row (get-last-console-line)))

    (if (get-attribute-value '<fear> (player.temp-attrs player))
        (put-coloured-str! +term-orange+ "Afraid" column row)
        (put-coloured-str! +term-white+  "      " column row))
    ))

(defmethod print-confused ((variant icfp-variant) (player player) 
			   (setting icfp-bottom-row-locations))

  (let ((column (slot-value setting 'confused))
        (row (get-last-console-line)))

    (if (get-attribute-value '<confusion> (player.temp-attrs player))
        (put-coloured-str! +term-orange+ "Confused" column row)
        (put-coloured-str! +term-white+  "        " column row))
    ))



(defmethod print-extra-frame-content ((variant icfp-variant) (dungeon dungeon) (player player))
  nil)



(defmethod redraw-stuff ((variant icfp-variant) (dungeon dungeon) (player player))
  
  (when (= 0 *redraw*) (return-from redraw-stuff nil))

  (let ((retval nil)
	(pr-set nil)
        (bot-set nil))

    (when (bit-flag-set? *redraw* +print-extra+)
      (bit-flag-remove! *redraw* +print-extra+)
      (bit-flag-remove! *redraw* +print-cut+)
      (bit-flag-remove! *redraw* +print-stun+)
      (bit-flag-remove! *redraw* +print-hunger+)
      (bit-flag-remove! *redraw* +print-blind+)
      (bit-flag-remove! *redraw* +print-confused+)
      (bit-flag-remove! *redraw* +print-afraid+)
      (bit-flag-remove! *redraw* +print-poisoned+)
      (bit-flag-remove! *redraw* +print-state+)
      (bit-flag-remove! *redraw* +print-speed+)
      (bit-flag-remove! *redraw* +print-study+)
;;      (unless pr-set (setf pr-set (get-setting variant :basic-frame-printing)))

      (print-extra-frame-content variant dungeon player)
      (setf retval t))

    
    (when (bit-flag-set? *redraw* +print-cut+)
      (bit-flag-remove! *redraw* +print-cut+)
      (unless pr-set (setf pr-set (get-setting variant :basic-frame-printing)))
      ;;(print-cut variant player pr-set)
      (setf retval t))

    (when (bit-flag-set? *redraw* +print-stun+)
      (bit-flag-remove! *redraw* +print-stun+)
      (unless pr-set (setf pr-set (get-setting variant :basic-frame-printing)))
      ;;(print-stun variant player pr-set)
      (setf retval t))

    (when (bit-flag-set? *redraw* +print-hunger+)
      (bit-flag-remove! *redraw* +print-hunger+)
      ;; fix
      (setf retval t))

    (when (bit-flag-set? *redraw* +print-blind+)
      (bit-flag-remove! *redraw* +print-blind+)
      ;; fix
      (setf retval t))

    (when (bit-flag-set? *redraw* +print-confused+)
      (bit-flag-remove! *redraw* +print-confused+)
      (unless bot-set (setf bot-set (get-setting variant :bottom-row-printing)))
      ;;(print-confused variant player bot-set)
      (setf retval t))

    (when (bit-flag-set? *redraw* +print-afraid+)
      (bit-flag-remove! *redraw* +print-afraid+)
      (unless bot-set (setf bot-set (get-setting variant :bottom-row-printing)))
      ;;(print-afraid variant player bot-set)
      (setf retval t))
    

    (when (bit-flag-set? *redraw* +print-poisoned+)
      (bit-flag-remove! *redraw* +print-poisoned+)
      (unless bot-set (setf bot-set (get-setting variant :bottom-row-printing)))
      ;;(print-poisoned variant player bot-set)
      (setf retval t))

    ;; state and speed in engine

    (when (bit-flag-set? *redraw* +print-study+)
      (bit-flag-remove! *redraw* +print-study+)
      ;; fix
      (setf retval t))


    (if (call-next-method)
	t
	retval)
    ))

(defmethod print-basic-frame ((variant icfp-variant) dungeon player)

  (declare (ignore dungeon))

  (put-coloured-str! +term-l-green+ "You are:" 0 2)

  (put-coloured-str! +term-yellow+ (format nil "Robot #~s" (robot-id *self*)) 1 3)

  (put-coloured-str! +term-l-green+ "Your bid:" 0 5)
  ;; hack
  (c-prt-token! +term-white+ +token-empty-field+ 6 0)
  (if (zerop *bid*)
    ;; This is intended to be a different color.
    (put-coloured-str! +term-yellow+ (format nil "~s"
                                              (* *bid-sign* *fallback-bid*))
                       1 6)
    (put-coloured-str! +term-l-red+ (format nil "~s"
					    (* *bid-sign* *bid*))
                       1 6))

  (put-coloured-str! +term-l-green+ "Your money:" 0 8)
  (c-prt-token! +term-white+ +token-empty-field+ 9 0)
  (put-coloured-str! +term-yellow+ (format nil "~s" *money*) 1 9)

  (put-coloured-str! +term-l-green+ "Carrying:" 0 11)
  (c-prt-token! +term-white+ +token-empty-field+ 12 0)
  (put-coloured-str! +term-yellow+ (format nil "~s" *weight*) 1 12)

  (put-coloured-str! +term-l-green+ "Capacity:" 0 14)
  (put-coloured-str! +term-yellow+ (format nil "~s" *maxweight*) 1 15)

  (put-coloured-str! +term-l-green+ "Location:" 0 17)
  (put-coloured-str! +term-yellow+ (format nil "(~3d,~3d)"
                                           (robot-x *self*)
					   (decanonicalize-y (robot-y *self*)))
                     1 18)

  
  )
